<template>
    <v-dialog v-model="dialog" persistent width="600">
        <v-card>
            <v-toolbar color="primary" dark dense>
                <v-btn icon rounded @click="$emit('close')">
                    <v-icon small>mdi-close</v-icon>
                </v-btn>
                {{ title }}
                <v-spacer/>
                <v-btn :disabled="sendable" :loading="loading" dark text @click="sendMail">Send
                    <v-icon right>mdi-send-outline</v-icon>
                </v-btn>
            </v-toolbar>
            <v-card-text>
                <v-container>
                    <v-col>
                        <v-row cols="12" md="4" sm="6">
                            <v-combobox ref="mailTo" v-model="form.to" :items="tos" :rules="rules" item-value="id" label="To"
                                        multiple return-oject @input="makeObject">
                                <template v-slot:item="scope">
                                    <span>{{ scope.item.name }}</span>
                                </template>
                                <template v-slot:selection="scope" v-bind="scope.attrs"
                                          @click:close="scope.parent.selectItem(scope.item)">
                                    <v-chip small>
                                        <v-avatar v-if="scope.item.photo" left>
                                            <v-img :src="scope.item.photo"></v-img>
                                        </v-avatar>
                                        <v-avatar v-else class="accent white--text" left
                                                  v-text="scope.item.name.slice(0, 1).toUpperCase()"></v-avatar>
                                        {{ scope.item.name }} <{{ scope.item.email }}>
                                    </v-chip>
                                </template>
                            </v-combobox>
                        </v-row>

                        <v-row cols="12" md="4" sm="6">
                            <v-text-field ref="mailSubject" v-model="form.subject" :rules="rules"
                                          label="Subject"></v-text-field>
                        </v-row>
                        <v-row cols="12" md="4" sm="6">
                            <v-textarea ref="mailBody" v-model="form.body" :rules="rules" label="Body"></v-textarea>
                        </v-row>
                        <v-row class="mt-5" cols="12" md="4" sm="6">
                            <v-chip v-for="(attach, index) in form.attachments" :key="`${attach.id}-${attach.label}`"
                                    class="mx-1 my-1" close small @click:close="removeAttachment(index)">
                                <v-avatar left small>
                                    <v-icon color="orange" small>mdi-file-{{ attach.type }}</v-icon>
                                </v-avatar>
                                {{ attach.label }}
                            </v-chip>
                        </v-row>
                    </v-col>
                </v-container>
            </v-card-text>
        </v-card>
    </v-dialog>
</template>

<script>
export default {
    name: "ComposeEmail",
    props: {
        dialog: Boolean,
        order: {
            type: Object
        },
        title: {
            type: String,
            default: 'Compose E-mail'
        }
    },
    data() {
        return {
            loading: false,
            customers: [],
            form: {
                to: [],
                subject: '',
                body: '',
                attachments: [],
            },
            error: {
                to: [],
                subject: [],
                body: [],
                attachments: [],
            },
            rules: [],
            eventSubscription: null,
        }
    },
    watch: {
        async dialog(open) {
            if (open) {
                this.customers = (await this.getCustomers())
                if (this.order && this.order.customer) {
                    this.form.subject = `Order '${this.order.code.toUpperCase()}' of ${this.$options.filters.dateTime(this.order.created_at)}`
                    const customer = this.customers.find(c => c.id === this.order.customer.id)
                    if (customer) {
                        this.form.to = [{
                            id: customer.id,
                            name: customer.name,
                            email: customer.email,
                            photo: customer.photo_path
                        }]
                        this.form.attachments.push({
                            module: 'payment',
                            type: 'pdf-box',
                            id: this.order.id,
                            label: `${this.order.code} - ${this.order.created_at.substring(0, 10)}`
                        })
                    }
                }
                this.$nextTick(this.$refs.mailBody.focus)
            } else {
                this.form.to = []
                this.form.cc = []
                this.form.attachments = []
                this.form.subject = ''
                this.form.body = ''
            }
        }
    },
    computed: {
        tos() {
            return this.customers.map(t => ({
                id: t.id,
                name: t.name,
                email: t.email,
                photo: t.photo_path
            }))
        },
        sendable () {
            return Object.keys(this.form).some(s => !this.form[s])
        }
    },
    methods: {
        sendMail() {
            if (!this.form.subject) {
                this.showSnack({message: 'Please, provide a subject to this message.', type: 'warning'})
                this.$refs.mailSubject.focus()
                return
            } else if (!this.form.to.length) {
                this.showSnack({message: 'Please, provide 1 or more receivers of this message.', type: 'warning'})
                this.$refs.mailTo.focus()
                return
            } else if (!this.form.body.length) {
                this.showSnack({message: 'Please, provide a message body.', type: 'warning'})
                this.$refs.mailBody.editor.editorCommands.commands.exec.mcefocus()
                return
            }
            this._confirmDialog({
                message: 'Are you sure you want to send this mail?', confirm: () => {
                    this.loading = true
                    axios.post(this.$route('mail.send'), this.form)
                        .then(resp => resp.data)
                        .then(data => {
                            this.showSnack({message: 'Email Sent!'})
                            this.$emit('close')
                        })
                        .catch(err => {
                            this.showSnack({message: err.response.data, type: 'error'})
                        })
                        .then(() => (this.loading = false))
                }
            })

        },
        getCustomers() {
            return new Promise((resolve, reject) => {
                axios.get(this.$route('customer.all'))
                    .then(resp => resolve(resp.data.data))
                    .catch(reject)
            })
        },
        makeObject(val) {
            let tos = Object.assign([], val)
            tos.forEach((to, index) => {
                if (typeof to === "string") {
                    if (/^[\w-\.]+@([\w-]+\.)+[\w-]{2,}$/.test(to)) {
                        this.form.to.splice(index, 1, {
                            email: to,
                            name: to.substr(0, to.indexOf('@')),
                            photo: ''
                        })
                    } else {
                        this.form.to.splice(index, 1)
                    }
                }
            })

        },
        removeAttachment(index) {
            this.form.attachments.splice(index, 1)
        },
        validInput() {
            for (const v of ['from', 'to', 'subject', 'body']) {
                if (!this.form[v].length) {
                    this.notify(this.$t('supply.message.email_field_empty', {field: v}), this.$t('status.validation'), 'error')
                    console.log(this.$refs[`mail${v.charAt(0).toUpperCase().concat(v.substring(1))}`])
                    this.$nextTick(this.$refs[`mail${v.charAt(0).toUpperCase().concat(v.substring(1))}`].focus)

                    return false
                }
            }
            return true
        }
    },
    mounted() {
    }
}
</script>

<style>
.sb-header {
    z-index: 100;
}
</style>
